@@ -1,9 +1,9 @@
 //////////////////////////////////////////////////////////////////////////////
 // Name:        SVGAnimationElement.cpp
-// Purpose:     
+// Purpose:     Implementation of wxSVGAnimationElement
 // Author:      Alex Thuering
 // Created:     2005/05/10
-// RCS-ID:      $Id: SVGAnimationElement.cpp,v 1.3 2013/09/12 08:42:50 ntalex Exp $
+// RCS-ID:      $Id: SVGAnimationElement.cpp,v 1.6 2014/03/27 08:42:16 ntalex Exp $
 // Copyright:   (c) 2005 Alex Thuering
 // Licence:     wxWindows licence
 //////////////////////////////////////////////////////////////////////////////
@@ -29,11 +29,13 @@ void wxSVGAnimationElement::ApplyAnimation() {
 		return;
 	if (GetTargetElement() == NULL)
 		SetTargetElement((wxSVGElement*) GetParent());
-	if (GetCurrentTime() >= GetStartTime() + GetDur())
-		GetTargetElement()->SetAttribute(GetAttributeName(), m_to.GetValueAsString());
-	else if (GetCurrentTime() >= GetStartTime()) {
-		wxSVGLength val(m_to.GetUnitType(), GetFrom().GetValue()
-				+ (GetTo().GetValue() - GetFrom().GetValue())*(GetCurrentTime() - GetStartTime())/GetDur());
-		GetTargetElement()->SetAttribute(GetAttributeName(), val.GetValueAsString());
+	if (GetCurrentTime() >= GetStartTime() + GetDur()) {
+		GetTargetElement()->SetAnimatedValue(GetAttributeName(), GetTo());
+	} else if (GetCurrentTime() >= GetStartTime() && GetFrom().GetPropertyType() == wxSVG_ANIMATED_LENGTH
+			&& GetTo().GetPropertyType() == wxSVG_ANIMATED_LENGTH) {
+		wxSVGAnimatedType value(wxSVGLength(GetTo().GetLength().GetUnitType(), GetFrom().GetLength().GetValue()
+				+ (GetTo().GetLength().GetValue() - GetFrom().GetLength().GetValue())*
+				(GetCurrentTime() - GetStartTime())/GetDur()));
+		GetTargetElement()->SetAnimatedValue(GetAttributeName(), value);
 	}
 }
