@@ -3,7 +3,7 @@
 // Purpose:     wxSVGCanvas - Base class for SVG renders (backends)
 // Author:      Alex Thuering
 // Created:     2005/05/04
-// RCS-ID:      $Id: SVGCanvas.cpp,v 1.22 2013/09/12 08:46:38 ntalex Exp $
+// RCS-ID:      $Id: SVGCanvas.cpp,v 1.25 2014/05/24 13:03:49 ntalex Exp $
 // Copyright:   (c) 2005 Alex Thuering
 // Licence:     wxWindows licence
 //////////////////////////////////////////////////////////////////////////////
@@ -124,6 +124,13 @@ void wxSVGCanvas::DrawCanvasText(wxSVGCanvasText& canvasText,
   }
 }
 
+wxSVGPatternElement* wxSVGCanvas::GetPatternElement(const wxSVGSVGElement& svgElem, const wxString& href) {
+	if (href.length() == 0 || href[0] != wxT('#') || &svgElem == NULL)
+	    return NULL;
+	wxSVGElement* elem = (wxSVGElement*) svgElem.GetElementById(href.substr(1));
+	return elem != NULL && elem->GetDtd() == wxSVG_PATTERN_ELEMENT ? (wxSVGPatternElement*) elem : NULL;
+}
+
 wxSVGGradientElement* wxSVGCanvas::GetGradientElement(const wxSVGSVGElement& svgElem, const wxString& href) {
 	if (href.length() == 0 || href[0] != wxT('#') || &svgElem == NULL)
 	    return NULL;
@@ -288,6 +295,8 @@ void wxSVGCanvas::RenderElement(wxSVGElement* elem, const wxSVGRect* rect, const
 		if (element->GetHeight().GetAnimVal().GetUnitType() == wxSVG_LENGTHTYPE_UNKNOWN)
 			((wxSVGAnimatedLength&) element->GetHeight()).SetAnimVal( wxSVGLength(wxSVG_LENGTHTYPE_PERCENTAGE, 100));
 		element->UpdateMatrix(matrix);
+		style.Add(element->GetStyle());
+		style.Add(element->GetAnimStyle());
 		if (rect && element->GetParent()) {
 			wxSVGRect rect2 = *rect;
 			wxSVGElement* parent = (wxSVGElement*) element->GetParent();
@@ -305,6 +314,7 @@ void wxSVGCanvas::RenderElement(wxSVGElement* elem, const wxSVGRect* rect, const
 			break;
 		element->UpdateMatrix(matrix);
 		style.Add(element->GetStyle());
+		style.Add(element->GetAnimStyle());
 		RenderChilds(elem, rect, &matrix, &style, ownerSVGElement, viewportElement, progressDlg);
 		break;
 	}
@@ -314,6 +324,7 @@ void wxSVGCanvas::RenderElement(wxSVGElement* elem, const wxSVGRect* rect, const
 			break;
 		element->UpdateMatrix(matrix);
 		style.Add(element->GetStyle());
+		style.Add(element->GetAnimStyle());
 		DrawLine(element, &matrix, &style);
 		break;
 	}
@@ -323,6 +334,7 @@ void wxSVGCanvas::RenderElement(wxSVGElement* elem, const wxSVGRect* rect, const
 			break;
 		element->UpdateMatrix(matrix);
 		style.Add(element->GetStyle());
+		style.Add(element->GetAnimStyle());
 		DrawPolyline(element, &matrix, &style);
 		break;
 	}
@@ -332,6 +344,7 @@ void wxSVGCanvas::RenderElement(wxSVGElement* elem, const wxSVGRect* rect, const
 			break;
 		element->UpdateMatrix(matrix);
 		style.Add(element->GetStyle());
+		style.Add(element->GetAnimStyle());
 		DrawPolygon(element, &matrix, &style);
 		break;
 	}
@@ -341,6 +354,7 @@ void wxSVGCanvas::RenderElement(wxSVGElement* elem, const wxSVGRect* rect, const
 			break;
 		element->UpdateMatrix(matrix);
 		style.Add(element->GetStyle());
+		style.Add(element->GetAnimStyle());
 		DrawRect(element, &matrix, &style);
 		break;
 	}
@@ -350,6 +364,7 @@ void wxSVGCanvas::RenderElement(wxSVGElement* elem, const wxSVGRect* rect, const
 			break;
 		element->UpdateMatrix(matrix);
 		style.Add(element->GetStyle());
+		style.Add(element->GetAnimStyle());
 		DrawCircle(element, &matrix, &style);
 		break;
 	}
@@ -359,6 +374,7 @@ void wxSVGCanvas::RenderElement(wxSVGElement* elem, const wxSVGRect* rect, const
 			break;
 		element->UpdateMatrix(matrix);
 		style.Add(element->GetStyle());
+		style.Add(element->GetAnimStyle());
 		DrawEllipse(element, &matrix, &style);
 		break;
 	}
@@ -368,6 +384,7 @@ void wxSVGCanvas::RenderElement(wxSVGElement* elem, const wxSVGRect* rect, const
 			break;
 		element->UpdateMatrix(matrix);
 		style.Add(element->GetStyle());
+		style.Add(element->GetAnimStyle());
 		DrawPath(element, &matrix, &style);
 		break;
 	}
@@ -379,6 +396,7 @@ void wxSVGCanvas::RenderElement(wxSVGElement* elem, const wxSVGRect* rect, const
 			break;
 		element->UpdateMatrix(matrix);
 		style.Add(element->GetStyle());
+		style.Add(element->GetAnimStyle());
 		DrawText(element, &matrix, &style);
 		break;
 	}
@@ -388,6 +406,7 @@ void wxSVGCanvas::RenderElement(wxSVGElement* elem, const wxSVGRect* rect, const
 			break;
 		element->UpdateMatrix(matrix);
 		style.Add(element->GetStyle());
+		style.Add(element->GetAnimStyle());
 		DrawImage(element, &matrix, &style, rect, progressDlg);
 		break;
 	}
@@ -397,6 +416,7 @@ void wxSVGCanvas::RenderElement(wxSVGElement* elem, const wxSVGRect* rect, const
 			break;
 		element->UpdateMatrix(matrix);
 		style.Add(element->GetStyle());
+		style.Add(element->GetAnimStyle());
 #ifdef USE_LIBAV
 		DrawVideo(element, &matrix, &style);
 #else
@@ -433,6 +453,7 @@ void wxSVGCanvas::RenderElement(wxSVGElement* elem, const wxSVGRect* rect, const
 			break;
 		element->UpdateMatrix(matrix);
 		style.Add(element->GetStyle());
+		style.Add(element->GetAnimStyle());
 		// test if visible
 		if (element->GetWidth().GetAnimVal().GetUnitType() != wxSVG_LENGTHTYPE_UNKNOWN
 				&& element->GetHeight().GetAnimVal().GetUnitType() != wxSVG_LENGTHTYPE_UNKNOWN) {
